/**
  * @file THERMO5_Click_FT90x.c
  * @brief This module is used to measure temperature
  *
  * @author Stefan Popovic
  *
  * @date 16/5/2017
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: FT900
  * Dev.Board: EasyFT90x v7
  * Oscillator: 100 MHz external
  * Ext. Modules: Thermostat Click Board
  * SW: MikroC PRO for FT90x v2.2.0

  * NOTES:
    - Place THERMO5 click board on mikroBUS slot 1
  /*****************************************************************************
  * Module Preprocessor Constants
  *****************************************************************************/
  #define THERMO5_ADDRESS   0x4C

  #define INT_DIODE      0x0029
  #define EXT_DIODE1     0x0110
  #define EXT_DIODE2     0x2324
  #define EXT_DIODE3     0x2A2B

  /*****************************************************************************
  * Module Preprocessor Macros
  *****************************************************************************/


  /*****************************************************************************
  * Module Typedefs
  *****************************************************************************/


  /*****************************************************************************
  * Includes
  *****************************************************************************/
  #include "built_in.h"
  #include "resources.h"
  #include <stdint.h>
  #include <stdbool.h>

  /*****************************************************************************
  * Module Variable Definitions
  *****************************************************************************/
  // Variables
  char tmp_data[10];
  char temp[2];
  char result;
  char txt[15];
  float diodeTemperature;
  
  void DrawFrame()
   {
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Fill_Screen(CL_WHITE);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("THERMO5 click", 80, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyFT90x v7", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Internal Diode:", 10, 60 );
    TFT_Write_Text("External Diode 1:", 10, 100 );
    TFT_Write_Text("External Diode 2:", 10, 140 );
    TFT_Write_Text("External Diode 3:", 10, 180 );
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Set_Pen( CL_WHITE , 1 );
  }

  float TH5Read(unsigned long diodeReg)
  {
       unsigned error;
       unsigned int Temperature;
       float Temp_Output;
       char Diode_Reg_Low, Diode_Reg_High;
       char Temp_Low_Byte, Temp_High_Byte;
       int8_t read_temp[2] = {0};
       char status_ = 0;

       Diode_Reg_Low = Lo(diodeReg);
       Diode_Reg_High = Hi(diodeReg);


       tmp_data[0] = Diode_Reg_Low;

       // Set slave address
       I2CM1_Set_Slave_Address(THERMO5_ADDRESS);

       status_ = I2CM1_Write_Bytes(&tmp_data[0], 1);

       status_ = I2CM1_Read_Bytes(tmp_data, 1);

       Temp_Low_Byte = tmp_data[0];

       tmp_data[0] = Diode_Reg_High;

       // Set slave address
       I2CM1_Set_Slave_Address(THERMO5_ADDRESS);

       status_ = I2CM1_Write_Bytes(&tmp_data[0], 1);

       status_ = I2CM1_Read_Bytes(tmp_data, 1);

       Temp_High_Byte = tmp_data[0];

       Temperature = (Temp_High_Byte << 8) | (Temp_Low_Byte);

       Temperature = Temperature >> 5;

       Temp_Output = Temperature * 0.125;

       return Temp_Output;
    }
    
  // Main program
  void main()
  {
   // TFT Init
   TFT_Init_ILI9341_8bit(320, 240);                      // TFT Initialization
   DrawFrame();

   I2CM1_Init(_I2CM_SPEED_MODE_STANDARD, _I2CM_SWAP_DISABLE);
   delay_ms(500);
   I2CM_Set_Active(&I2CM1_Soft_Reset,
                   &I2CM1_Set_Slave_Address,
                   &I2CM1_Write, &I2CM1_Write_10Bit,
                   &I2CM1_Write_Bytes,
                   &I2CM1_Read,
                   &I2CM1_Read_10Bit,
                   &I2CM1_Read_Bytes);

   //***************************************************************************
   // Temperature read loop
   while (1)
   {
      TFT_Rectangle( 200, 50, 280, 200 );

      diodeTemperature = TH5Read(INT_DIODE);
      sprintf( txt, "%.2f C", diodeTemperature);
      TFT_Write_Text( txt, 200, 60 );

      diodeTemperature = TH5Read(EXT_DIODE1);
      sprintf( txt, "%.2f C", diodeTemperature);
      TFT_Write_Text( txt, 200, 100 );

      diodeTemperature = TH5Read(EXT_DIODE2);
      sprintf( txt, "%.2f C", diodeTemperature);
      TFT_Write_Text( txt, 200, 140 );

      diodeTemperature = TH5Read(EXT_DIODE3);
      sprintf( txt, "%.2f C", diodeTemperature);
      TFT_Write_Text( txt, 200, 180 );

      delay_ms(1000);
   }
   //***************************************************************************
}



